
architecture dataflow of hazard_fsm is

signal current_state : fsm_states;
signal next_state    : fsm_states;

begin


  --------------------------------------------------------------------------
  -- State switch
  --------------------------------------------------------------------------
  state_switch : process(phi2)
  begin
    --
    -- switch to next state with rising edge of phi1
    --
    if (phi2'event and phi2 = '0') then
      if reset = '1' then
	current_state <= reset_1;
      else
	current_state <= next_state;
      end if;
    end if;
  end process state_switch;


  --------------------------------------------------------------------------
  -- Next state logic
  -- S' = f (X,S)
  --------------------------------------------------------------------------
  next_state_logic : process (current_state, branch_in, fetch_memory_not_ready,
                              adr_ls2, memory_not_ready)

  begin
    case current_state is
      ----------------------------------------------------------------
      when reset_1 =>
	next_state <= reset_2;
      ----------------------------------------------------------------
      when reset_2 =>
	next_state <= fetch;
      ----------------------------------------------------------------
      when fetch =>
        if forward_from_memory = '1' then
         next_state <= stop_fetch_dec_exec;
    	elsif branch_in = '0' then
         next_state <= branch_1;
        elsif fetch_memory_not_ready = '1' then
         next_state <= re_fetch;
        else
         next_state <= fetch;
        end if;
     ----------------------------------------------------------------
      when re_fetch =>
        if fetch_memory_not_ready = '0' then
         next_state <= fetch;
        else
         next_state <= re_fetch;
        end if;
      ---------------------------------------------------------------
      when stop_fetch_dec_exec =>
        if memory_not_ready = '1' then
         next_state <= stop_fetch_dec_exec;
        else
          if fetch_memory_not_ready = '1' then -- IR not valid
            next_state <= re_fetch;
          else
            next_state <= fetch;
          end if;
        end if;
      ---------------------------------------------------------------
      when branch_2 =>
 	next_state <= branch_3;
     ----------------------------------------------------------------
      when branch_1 =>
	if (adr_ls2(1) = '1' or adr_ls2(0) = '1') then  -- iav ?
-- 	  if super = '0' then
	    next_state <= i_adr_viol;
--	  else
--	    next_state <= error_state;
--	  end if; 
--	elsif adr_msb  = '0' and super = '0' then           -- priv ?
-- 	  if super = '0' then
--	    next_state <= priv_viol;
--	  else
--	    next_state <= error_state;
--	  end if;  
	else
          next_state <= branch_2;
	end if;   	
      ----------------------------------------------------------------
      when branch_3 =>
 	next_state <= fetch;
      ----------------------------------------------------------------
      when i_adr_viol =>
        next_state <= branch_2;
     end case;
  end process next_state_logic;
  
  ----------------------------------------------------------------
  -- Output logic block
  -- Y = f(S) 
  ----------------------------------------------------------------
  output_logic : process (current_state)
  begin
    case current_state is
      ----------------------------------------------------------------
      when reset_1 =>
        stall_fetch <= '0';
        stall_decode <= '0';
        stall_exec <= '0';    
      ----------------------------------------------------------------
      when reset_2 =>
        stall_fetch <= '1'; -- Hier schon '1' setzen ???
	stall_decode <= '0';	
      ----------------------------------------------------------------
      when fetch =>
        stall_fetch <= '1';
        stall_decode <= '1';
        stall_exec <= '1';
      ----------------------------------------------------------------
      when re_fetch =>
        stall_fetch <= '1';
        stall_decode <= '0';
      ----------------------------------------------------------------
      when branch_1 =>
        stall_fetch  <= '0';
	stall_decode <= '0';	
      ----------------------------------------------------------------
      when branch_2 =>
        stall_fetch <= '1'; -- PC <- MAR
	stall_decode <= '0';
        force_decode <= nop;	
      ----------------------------------------------------------------
      when branch_3 =>
        stall_fetch <= '1'; -- oder 0 ?
	stall_decode <= '0';	
      ----------------------------------------------------------------
      when stop_fetch_dec_exec =>
        stall_fetch <= '0';
        stall_decode <= '0';
        stall_exec <= '0';
      ----------------------------------------------------------------
      when i_adr_viol =>
        stall_decode <= '0';
        force_decode <= iav;
      ----------------------------------------------------------------
     end case;
  end process output_logic;

end dataflow;
