--------------------------------------------------------------------------
--  DLX PROCESSOR MODEL SUITE
--  Copyright (C) 1995, Martin Gumm
--  University of Stuttgart / Department of Computer Science / IPVR-ISE
--------------------------------------------------------------------------
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 1, or (at your option)
--  any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--------------------------------------------------------------------------
--  Last revision date : November 15 1995
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--  *** SYNOPSYS synthesizable code (ver. 3.2.a) ***
--------------------------------------------------------------------------

--------------------------------------------------------------------------
--   behaviour architecture for memory data register (32-bit)
--   with one tri-state output and one normal output 
--  (register with special shift capabilities)
--
--  (file mdr-behaviour.vhd)
--------------------------------------------------------------------------

architecture behaviour of mdr2 is

  signal master : dlx_word;
  signal slave : dlx_word;
  signal shift_ctrl_m, shift_ctrl_s : std_logic_vector(0 to 2);
  signal op_vec : std_logic_vector(0 to 4);
  constant dc_16 : dlx_half := "----------------";
  constant dc_8 : dlx_byte := "--------";



begin

  op_vec <= shift_ctrl_s & mar_ls2_in;

  latch : process (d, latch_en)
  begin
    if latch_en = '0' then
      master <= To_UX01(d) after tpd_latch;
      shift_ctrl_m <= shift_ctrl after tpd_latch;
    end if;
    if latch_en = '1' then 
      slave <= master after tpd_latch;
      shift_ctrl_s <= shift_ctrl_m after tpd_latch;
    end if;
  end process latch;
  
  logic : process (slave, op_vec)
  begin
    case op_vec is
      when "10010" =>                          -- load ls-half
        -- load lower half into upper half
        q(0 to 15) <= std_logic_vector'(0 to 15 => slave(16)) after tpd_logic;
        q(16 to 31) <= slave(16 to 31) after tpd_logic;
      when "10110" =>                         -- load byte 2
        -- (load byte 2 into byte 0)
	q(0 to 23) <= std_logic_vector'(0 to 23 => slave(16)) after tpd_logic;
        q(24 to 31) <= slave(16 to 23) after tpd_logic;
      when "10101" =>                         -- load byte 1
	-- load byte 1 into byte 0
	q(0 to 23) <= std_logic_vector'(0 to 23 => slave(8)) after tpd_logic;
        q(24 to 31) <= slave(8 to 15) after tpd_logic;
      when "10111" =>                         -- load byte 3
	-- load byte 1 into byte 0
	q(0 to 23) <= std_logic_vector'(0 to 23 => slave(24)) after tpd_logic;
        q(24 to 31) <= slave(24 to 31) after tpd_logic;
      when "11000" =>                         -- store ms-half
	-- load lower half into upper half
        q <= To_UX01(slave(16 to 31)) & dc_16 after tpd_logic;
      when "11101" =>                         -- store byte 1
	-- (load byte 3 into byte 1)
        q <= dc_8 & To_UX01(slave(24 to 31)) & dc_16 after tpd_logic;
      when "11100" =>                         -- store byte 0
	-- load byte 3 into byte 0
	q <= To_UX01(slave(24 to 31)) & dc_8 & dc_16 after tpd_logic;
      when "11110" =>                         -- store byte 2
	-- load byte 3 into byte 2
	q <= dc_16 & To_UX01(slave(24 to 31)) & dc_8 after tpd_logic;
      when "11010" =>                         -- store ls half
        q <= dc_16 & To_UX01(slave(16 to 31)) after tpd_logic;
      when "10000" =>                         -- load ms half
        q(0 to 15) <= std_logic_vector'(0 to 15 => slave(0)) after tpd_logic;
        q(16 to 31) <= slave(0 to 15) after tpd_logic;
      when "11111" =>	                      -- store byte 3
        q <= dc_16 & dc_8 & slave(24 to 31) after tpd_logic;
      when "10100" =>
        q(0 to 23) <= std_logic_vector'(0 to 23 => slave(0)) after tpd_logic;
        q(24 to 31) <= slave(0 to 7) after tpd_logic;
      when others =>                         -- load word / load byte 0
	                                     -- store word
        -- load unchanged from input
        q <= To_UX01(slave) after tpd_logic;
    end case;
  end process logic;

  
end behaviour;





