
architecture structural of pipe_control is

component hazard_fsm
 port( phi2 : in std_logic;
       reset  : in std_logic;
       --
       adr_ls2 : in std_logic_vector(0 to 1);
       branch_in : in std_logic;
       fetch_memory_not_ready : in std_logic;
       memory_not_ready       : in std_logic;
       forward_from_memory    : in std_logic;
       --
       stall_fetch  : out std_logic;
       stall_decode : out std_logic;
       stall_exec   : out std_logic;
       force_decode : out exc_type);
end component;

component latch 
 generic(tpd_latch : Time);
 port (d : in std_logic;
       q : out std_logic;
       latch_en : std_logic);
end component;

component forward_logic
generic(tpd_latch : Time;
        tpd_comp  : Time;
        tpd_mux   : Time);
port (phi2         : in std_logic;
      rd_valid     : in std_logic; -- rd a valid address ?
      --  issued       : in std_logic; -- instr issued ?
      rd_shift     : in std_logic;
      rs1, rs2, rd : in std_logic_vector(0 to 4);
      --
      mux1_out, mux2_out : out std_logic_vector(0 to 1);
      branch_reg_mux : out std_logic_vector(0 to 1);
      forward_from_memory : out std_logic;
      --
      load_instr : in std_logic);
end component;

signal not_phi2 : std_logic;
signal not_phi1 : std_logic;

signal intrn_stall_fetch : std_logic;
signal intrn_stall_decode : std_logic;
signal intrn_stall_exec : std_logic;

signal forward_from_memory : std_logic;
signal instr_stop : std_logic;
signal rd_valid : std_logic;

signal re_fetch : std_logic;


begin

hazard_handler : hazard_fsm
 port map (phi2 => phi2,
           reset => reset,
           --
           adr_ls2 => adr_ls2,
           branch_in => branch,
           fetch_memory_not_ready => fetch_memory_not_ready,
           memory_not_ready       => mem_memory_not_ready,
           forward_from_memory => forward_from_memory,
           --
           stall_fetch  => intrn_stall_fetch,
           stall_decode => intrn_stall_decode,
           stall_exec => intrn_stall_exec,
           force_decode => force_decode);

forwarding : forward_logic
 generic map(tpd_latch => 2 ns, tpd_comp => 2 ns, tpd_mux => 2 ns)
 port map (phi2 => phi2,
           rd_valid => rd_valid,
           rd_shift => phi1,
           rs1 => rs1,
           rs2 => rs2,
           rd => rd,
           mux1_out => mux1_out,
           mux2_out => mux2_out,
           branch_reg_mux => branch_reg_mux,
           forward_from_memory => forward_from_memory,
           --
           load_instr => load_instr);

not_phi2 <= not phi2;
not_phi1 <= not phi1;

stall_fetch <= intrn_stall_fetch;
stall_decode <= intrn_stall_decode; -- nop eingeschoben
stall_exec <=  intrn_stall_exec; -- nop eingeschoben

rd_valid <= load_instr or reg_dest;





end structural;



